#include <stdint.h>
#include <stdbool.h>

// C_Meter Click
sbit C_METER_RST        at RC1_bit;
sbit C_METER_RST_DIR    at TRISC1_bit;
sbit C_METER_INT        at RE8_bit;
sbit C_METER_INT_DIR    at TRISE8_bit;

void system_setup( void );
void setup_interrupt( void );
void InitTimer1( void );

char uart_text[ 40 ];
char final_text[ 40 ];
uint32_t int_count = 0;
uint32_t timer_count_end = 0;
bool sec_flag = false;
float c_cal = 0.0;
float t_meas = 0.0;
float c_meas = 0.0;
float final = 0.0;
float t_cal = 0.0;


void main()
{
    //Local Declarations
    bool calibration_flag = true;


    system_setup();
    setup_interrupt();
    InitTimer1();

    t_cal = 1.0 / 17776.0;
    c_cal = t_cal / 77616.0;

    int_count = 0;

    while(1)
    {

        if( sec_flag )
        {
            //Debugging
            sprintf( uart_text, "Interrupts : %u\r\n", timer_count_end );
            UART2_Write_Text( uart_text );
            //Measurment
            t_meas = 1.0 / timer_count_end;
            c_meas = t_meas / 77616.0;
            final = fabs( c_cal - c_meas );
            //Print out
            UART2_Write_Text( "Calibrated: " );
            FloatToStr( c_cal, uart_text );
            UART2_Write_Text( uart_text );
            UART2_Write_Text( "\r\n" );
            UART2_Write_Text( "Final: " );
            FloatToStr( final, final_text );
            UART2_Write_Text( final_text );
            UART2_Write_Text( "\r\n" );
            //Reset flags
            sec_flag = false;
            int_count = 0;
            timer_count_end = 0;
        }
    }
}

void system_setup( void )
{
    //GPIO
    C_METER_INT_DIR = 0;
    C_METER_RST_DIR = 1;

    //RST Toggle
    C_METER_RST = 0;
    Delay_ms(50);
    C_METER_RST = 1;

    //UART
    UART2_Init( 9600 );
    UART2_Write_Text( "UART Initialized\r\n" );

}

void setup_interrupt( void )
{
  AD1PCFG    = 0xFFFF;               // Turning all analog pins to digital
  TRISE8_bit = 1;                    // Set RE9 pin as input
  TRISB      = 0;                    // Set PORTB as output
  LATB       = 0;                    // Clear PORTB

  LATB = 0x5555;                     // Set PORTB start value

  INT1IP0_bit = 0;                   // Set INT2 interrupt
  INT1IP1_bit = 0;                   // Set interrupt priorities
  INT1IP2_bit = 1;                   // Set inrrupt priority to 4
  INT1IE_bit  = 1;                   // Set interrupt on INT2 (RE9) to be enabled
  EnableInterrupts();                // Enable interruts as previously set
}

void int1_interrupt() iv IVT_EXTERNAL_1 ilevel 4 ics ICS_AUTO {
  int_count++;
  INT1IF_bit = 0;                    // Reset INT1 flag
}

void InitTimer1(){
  T1CON         = 0x8030;
  T1IP0_bit         = 1;
  T1IP1_bit         = 1;
  T1IP2_bit         = 1;
  T1IF_bit         = 0;
  T1IE_bit         = 1;
  PR1                 = 62500;
  TMR1                 = 0;
}
void Timer1Interrupt() iv IVT_TIMER_1 ilevel 7 ics ICS_SRS 
{
  static count = 0;
  
  T1IF_bit         = 0;
  //Enter your code here
  count++;
  if( count >= 4 )
  {
      timer_count_end = int_count;
      int_count = 0;
      sec_flag = true;
      count = 0;
  }
}